/* $Id: et6000_io.c,v 1.4 1998/11/24 18:28:10 marcus Exp $
******************************************************************************

   Tseng Labs ET6000 I/O

   Copyright (C) 1997 Jim Ursetto    [jim.ursetto@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

	This code was derived from the following source of information:
	[1]	ET4000 driver (c) Jason McMullan and Andreas Vogler
	[2]	Prerelease VGADOC5 by Finn Thoegersen

******************************************************************************
*/

#include <IBM/vga.h>
#include <Tseng/et6000.h>
#include <Tseng/et6000_io.h>
#include <kgi/system.h>

/* !! FIXME */
#define PCI_IO (et6000_pci_io.baseptr)

/*
	generalized DAC output functions
*/

void enable_ext(void)
{
	io_out8(0x03, HERC_CMPT);   /* from et6000.h */
	io_out8(0xa0, DISPLAY_MODE);
}

void disable_ext(void)
{
	io_out8(0x29, DISPLAY_MODE);
	io_out8(0x01, HERC_CMPT);
}

static kgiu16 dac_port[4] ={ 0x3C8, 0x3C9, 0x3C6, 0x3C7 };

void DAC_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 dac_reg)
{
	if (dac_reg & ~0x3)
		printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
			   __FILE__, dac_reg, dac_reg &= 0x3);
    
	io_out8(value, dac_port[dac_reg & 0x03]);
}

kgiu8 DAC_in8(struct kgi_display *dpy,kgiu8 dac_reg)
{
	if(dac_reg & ~0x03) 
		printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
		        __FILE__, dac_reg, dac_reg &= 0x03);
    
	return io_in8(dac_port[dac_reg & 0x03]);
}

void DAC_ins8(struct kgi_display *dpy,kgiu8 dac_reg,void *buf, uint count)
{
	if(dac_reg & ~0x03)
		printk("%s: warning; DAC register select lines truncated: %x to %x.\n",\
			   __FILE__, dac_reg, dac_reg &= 0x3);
   
	io_ins8(dac_port[dac_reg & 0x03], buf, count);
}

void DAC_outs8(struct kgi_display *dpy,kgiu8 dac_reg, void *buf, uint count)
{
	if(dac_reg & ~0x03) 
		printk("%s: warning: DAC register select lines truncated: %x to %x.\n",\
			   __FILE__, dac_reg, dac_reg &= 0x03);
    
	printk("DAC_outs8\n");
	return;
	io_outs8(dac_port[dac_reg & 0x03], buf, count);
}

/*	clock I/O functions
*/

/* Selects a specific clock.  Doesn't change value of clock timing! */
void CLK_out8(struct kgi_display *dpy,kgiu8 value,kgiu8 reg)
{
	if (reg!=0)
		INTERNAL_ERROR;
	PASSERT(!(value & ~0x07), "trying to select clock: %x", value);
	DEBUG2("selecting clock #%d\n", value);
	enable_ext();
	MISC_out8((MISC_in8() & ~MISC_CLOCK_MASK) | ((value & 0x03) << 2));
	CRT_out8((CRT_in8(CRT_CCR) & ~CR34_CLK_SEL2) | ((value & 0x04) >> 1),
			CRT_CCR);
	/* ignore high bits */
	CRT_out8((CRT_in8(CRT_GENERAL) & ~CR31_CLK_MASK), CRT_GENERAL);
	DEBUG2("selected clock #%d\n", ((io_in8(MISCr) & MISC_CLOCK_MASK) >> 2)
		   | ((CRT_in8(CRT_CCR) & CR34_CLK_SEL2) << 1));
	disable_ext();			  
}

kgiu8 CLK_in8(struct kgi_display *dpy,kgiu8 reg)
{
	kgiu8 ret=0;
	if (reg!=0)
		INTERNAL_ERROR;
    
	ret |= (io_in8(MISCr) & MISC_CLOCK_MASK) >> 2;
	ret |= (CRT_in8(CRT_CCR) & CR34_CLK_SEL2) << 1;
	/* It is unknown whether or not index 31 of the CRT register
	 * should be returned. VGADOC is unclear. */
   
	return ret;
}

/* Should inline these. */
kgiu8 MISC_in8(void) {
	return io_in8(MISCr);
}

void MISC_out8(kgiu8 value)
{
	io_out8(value | MISC_COLOR_IO, MISCw);
}

kgiu8 CRT_in8(kgiu8 reg)
{ 
	io_out8(reg, CRTI);
	return io_in8(CRTD); 
}

kgiu8 SEQ_in8(kgiu8 reg)
{ 
	io_out8(reg, SEQI); 
	return io_in8(SEQD); 
}

kgiu8 GRC_in8(kgiu8 reg)
{ 
	io_out8(reg, GRCI); 
	return io_in8(GRCD); 
}

static kgiu8 ATC_in8(kgiu8 reg)
{ 
	io_out8(reg, ATCI); 
	return io_in8(ATCDr); 
}

void CRT_out8(kgiu8 value, kgiu8 reg) 
{ 
	outw((value << 8) | reg, CRTI); 
}

void SEQ_out8(kgiu8 value, kgiu8 reg)
{ 
	outw((value << 8) | reg, SEQI); 
}

void GRC_out8(kgiu8 value, kgiu8 reg)
{ 
	outw((value << 8 ) | reg, GRCI); 
}

void ATC_out8(kgiu8 value, kgiu8 reg)
{ 
	io_out8(reg, ATCI); 
	io_out8(value, ATCDw); 
}

void PCI_out8(struct kgi_display *dpy, kgiu8 value, kgiu8 reg)
{
	ASSERT(reg < NoPCIRegs);
	io_out8(value,PCI_IO+reg);
}

/*EXPORT_SYMBOL(PCI_out8);*/

kgiu8 PCI_in8(struct kgi_display *dpy, kgiu8 reg)
{
	ASSERT(reg < NoPCIRegs);
	return io_in8(PCI_IO+reg);
}

static void save_SEQ(kgiu8 *buf)
{
	int i;
    
	for(i = 0; i < 5; i++) 
		*(buf++)=SEQ_in8(i);
	buf++;
	for(i = 6; i < 8; i++) 
		*(buf++)=SEQ_in8(i);
}

static void restore_SEQ(kgiu8 *buf)
{
	register kgiu8 i;
	for(i=0; i < 5; i++) 
		SEQ_out8(*(buf++),i);
	buf++;
	for(i=6; i < 8; i++) 
		SEQ_out8(*(buf++),i);
}

static void save_CRT(kgiu8 *buf)
{
	int i;
    
	for(i = 0; i < 0x19; i++) 
		*(buf++)=CRT_in8(i);
	buf+=0x17;
	for(i = 0x30; i < 0x40; i++) 
		*(buf++)=CRT_in8(i);
}

static void restore_CRT(kgiu8 *buf)
{
	register kgiu8 i;

	CRT_out8(CRT_in8(CRT_VSYNCEND) & ~CR11_LOCKTIMING, CRT_VSYNCEND);
	CRT_out8(CRT_in8(CRT_HBLANKEND) | CR03_IS_VGA,CRT_HBLANKEND);
    
	for(i = 0; i < 0x19; i++) {
		switch (i) {
			case CRT_HBLANKEND:
				*buf |= CR03_IS_VGA;
				break;
			case CRT_VSYNCEND:
				*buf &= ~CR11_LOCKTIMING;
				break;
			default:
				break;
		}
		CRT_out8(*(buf++), i);
	}
	buf+=0x17;

	for(i = 0x30; i < 0x40; i++) {
		switch(i) {
			/*case CRT_GENERAL:
			 *buf = (io_in8(CRT_GENERAL) & CR31_CLK_MASK)
	         | (*buf & ~CR31_CLK_MASK);
			 break;*/
			case CRT_CCR:
				*buf = (CRT_in8(CRT_CCR) & CR34_CLK_SEL2) 
					| (*buf & ~CR34_CLK_SEL2);
				break;
			default:
				break;
		}
		CRT_out8(*(buf++), i);
	}
}

static void save_GRC(kgiu8 *buf)
{
	int i;

	for(i = 0; i < NoGRCRegs; i++) 
		*(buf++)=GRC_in8(i);
}

static void restore_GRC(kgiu8 *buf)
{
	register kgiu8 i;

	for(i = 0; i < NoGRCRegs; i++) 
		GRC_out8(*(buf++),i);
}

static void save_ATC(kgiu8 *buf)
{
	int i;
    
	for(i = 0; i < 0x15; i++) {
		io_in8(INS1r);  /* reset AFF and setup index */
		*(buf++)=ATC_in8(i);
	}
	buf++;
	for(i = 0x16; i < 0x18; i++) {
		io_in8(INS1r);  
		*(buf++)=ATC_in8(i);
	}
	io_in8(INS1r); 
	io_out8( ATCI_ENABLEDISPLAY, ATCI);
}

static void restore_ATC(kgiu8 *buf)
{
	register kgiu8 i;
    
	for(i = 0; i < 0x15; i++) {
		io_in8(INS1r); 
		ATC_out8(*(buf++),i);
	}
	buf++;
	for(i = 0x16; i < 0x18; i++) {
		io_in8(INS1r); 
		ATC_out8(*(buf++),i);
	}
	io_in8(INS1r); 
	io_out8(ATCI_ENABLEDISPLAY, ATCI);
}

static void restore_PCI(struct statetable *state)
{
	io_out8(state->PCIVidCtrl1, PCI_IO+PCI_VIDCTRL1);
}    

static void save_PCI(struct statetable *state)
{
	state->PCIVidCtrl1 = io_in8(PCI_IO+PCI_VIDCTRL1);
}    

void save_state(struct statetable *state)
{
	state->MISC  = io_in8(MISCr);	/* save miscellaneous output reg */

	save_SEQ(state->SEQ);	/* save sequencer regs */
	save_CRT(state->CRT);	/* save cathode ray tube controller regs */
	save_GRC(state->GRC);	/* save graphics controller regs */
	save_ATC(state->ATC);	/* save attribute controller regs */
	save_PCI(state);            /* save PCI regs */
}

void restore_state(struct statetable *state)
{
    /* restore_state is called as a mode set after the clock is
     * set in kgim_clock_set.  Do not set the clock here!! */
	MISC_out8((MISC_in8() & MISC_CLOCK_MASK) | state->MISC);
	enable_ext();
	restore_SEQ(state->SEQ);
	restore_CRT(state->CRT);
	restore_GRC(state->GRC);
	restore_ATC(state->ATC);
	restore_PCI(state);
	disable_ext();
}

void dump_state(struct statetable *state)
{ 
	int i;
	printk("MISC: %.2x",state->MISC);
  
	for(i = 0; i < NoSEQRegs; i++) {
		if(!(i & 0x7)) printk("\nSEQ:%.2x: ",i);
		printk("0x%.2x, ",state->SEQ[i]);
	}
	for(i = 0; i < NoCRTRegs; i++) {
		if(!(i & 0x7)) printk("\nCRT:%.2x: ",i);
		printk("0x%.2x, ",state->CRT[i]); 
	}
	for(i = 0; i < NoGRCRegs; i++) {
		if(!(i & 0x7)) printk("\nGRC:%.2x: ",i);
		printk("0x%.2x, ",state->GRC[i]); 
	}
	for(i = 0; i < NoATCRegs; i++) {
		if(!(i & 0x7)) printk("\nATC:%.2x: ",i);
		printk("0x%.2x, ",state->ATC[i]); 
	}
	printk("\n");
}
