/* $Id: mach64.h,v 1.4 1999/06/27 16:01:15 marcus Exp $
******************************************************************************

   ATI Mach64 Series -- global definitions and structures

   Copyright (C) 1997		Simon Hammett

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef	_MACH64_H__
#define	_MACH64_H__

#include "kgi/module.h"
/*
 * store useful info about mach64
 */

extern struct kgi_chipset Mach64_Chip_Info;


struct	cardInfo{

	int	Using_MMIO;		/* true if using LFB to access regs */
	int	VGA_Enabled;		/* true if vga should be enabled/used */
	int	*Registers;		/* Pointer to array of registers */
	kgiu32	LFB_Base;		/* Absolute Physical Address of LFB */
	void 	*LFB_VBase;		/* Virtual address of LFB */
	kgiu32	LFB_Size;		/* size of LFB may be != ramsize */
	void	*MMIO_Base;		/* Virtual address of MMIO regs.  */
	kgiu16	SIO_Base;		/* io base */
        kgiu32  Register_Base;		/* Something XFree has */
	kgiu32	Ram_Size;		/* number of bytes of ram. */
	kgiu16	Card_Type;		/* type of card 0 to MACH64_UNKNOWN */
	kgiu16	Revision;
	kgiu16	Dac_Type;
	char	Bus, Devfn;

/*---- Registers */

	kgiu32	crtc_h_total,
		crtc_h_sync,
		crtc_v_total,
		crtc_v_sync,
		crtc_vline,
		crtc_display_start,
		crtc_int_cntl,
		crtc_gen_cntl,
		clock_cntl,
		bus_cntl,
		mem_cntl,
		dac_cntl,
		config_cntl,
		gen_test_cntl;
 		struct cardInfo	*next;

};

struct devInfo {
    struct cardInfo *cards;
};


/* helps access the cardInfo structure from any kgim_ function. */
#define GET_PRIVATE \
        struct cardInfo *card = KGIM_PRIVATE(dpy, chipset)


#ifdef _MACH64_C__
    struct devInfo info;
#else
    extern struct devInfo info;

#endif
enum	{
		MACH64_GX,
		MACH64_CX,
		MACH64_CT,
		MACH64_ET,
		MACH64_VT,
		MACH64_VU,
		MACH64_GT,
		MACH64_GU,
                MACH64_GB,
                MACH64_GD,
                MACH64_GI,
		MACH64_GP,
                MACH64_GQ,
		MACH64_UNKNOWN
	};

enum	{
		RAMDAC_UNKNOWN,
		RAMDAC_IBMRGB_514,
		RAMDAC_ATI_68875,
		RAMDAC_BT_467_478,
		RAMDAC_BT_481,
		RAMDAC_ATI_68860_68880,
		RAMDAC_STG_1700,
		RAMDAC_SC_15021,
		RAMDAC_INTERNAL
	};
#ifdef _MACH64_DETECT_C__
	static	char	*Mach64_RAMDAC_Descriptions[]={
					"Unknown",
					"IbmRgb514",
					"ATI 68875",
					"Bt 476/478",
					"Bt 481",
					"ATI 68860/68880",
					"STG1700",
					"SC15021",
					"Internal"
				};

	static	kgiu16	Mach64_PCI_Ids[]={
					0x4758,		/* MACH64_GX */
					0x4358,		/* MACH64_CX */
					0x4354,		/* MACH64_CT */
					0x4554,		/* MACH64_ET */
					0x5654,		/* MACH64_VT */
					0x5655,		/* MACH64_VU */
					0x4754,		/* MACH64_GT */
					0x4755,		/* MACH64_GU */
                      			0x4742,		/* MACH64_GB */
					0x4744,		/* MACH64_GD */
					0x4749,		/* MACH64_GI */
					0x4750,		/* MACH64_GP */
					0x4751,		/* MACH64_GQ */
					0
				};

	static	int	Mach64_SIO_Bases[]={
					0x2ec, 0x1cc, 0x1c8
				};

	static	char	*Mach64_Descriptions[]={
					"Mach64 GX - Graphics Pro Turbo 1600",
					"Mach64 CX - Send Description",
					"Mach64 CT - Graphics Xpression",
					"Mach64 ET - Send Description",
					"Mach64 VT - Send Description",
					"Mach64 VU - Send Description",
					"Mach64 GT - Rage II - 3D Expression",
					"Mach64 GU - Rage II+DVD",
					"Mach64 GB - Rage Pro",
					"Mach64 GD - Rage Pro",
					"Mach64 GI - Rage Pro",
					"Mach64 GP - Rage Pro",
					"Mach64 GQ - Rage Pro",
					"BUG! Shouldn't See This"
				};

				
	static	struct mem_region Mach64_LFB_Mem =
	{
		MEM_NULL,
		0,
		8 MB,
		-1,
		"Mach64 Linear Frame Buffer"
	};

	
/* this struct is basically copied from the Matrox driver. */
static struct kgi_mmio_region graphic_mmio_linear_fb =
{
	NULL,				
	"Mach64 MMIO linear framebuffer",	/* Name */
	MMIO_FRAMEBUFFER,			/* Type */
	MMIO_RDWR | MMIO_SHARED,		/* Protection Map */
	0x0,					/* Total Size of Area */
	0x0,					/* Physical Size */
	0x0,					/* bus address of the windows */
	0,					/* Logical address */
	NULL,
	NULL
};
#else

extern struct kgi_mem_region Mach64_LFB_Mem;
extern struct kgi_mmio_region graphic_mmio_linear_fb;

#endif
#endif


